#include <iostream>
#include <vector>
#include <queue>

using namespace std;

typedef long long ll;

struct Edge {
    int u, v;
    ll d;
    Edge (int uu = 0, int vv = 0, ll dd = 0) : u(uu), v(vv), d(dd) {}
    bool operator<(const Edge& rhs) const { return d > rhs.d; }
};

vector<vector<Edge>> g;
vector<Edge> path; // path[v].u will be predecessor of u on v
vector<bool> marked;
vector<int> answer;

void dijkstra(int s) {
    path.resize(g.size(), Edge(-1, -1, -1));
    priority_queue<Edge> pq;
    pq.push(Edge(-1,s,0));

    while (!pq.empty()) {
        Edge curr = pq.top();
        pq.pop();
        if (path[curr.v].d != -1) continue;

        path[curr.v] = curr;
        for (auto nxt : g[curr.v])
            pq.push(Edge(nxt.u, nxt.v, nxt.d + curr.d));
    }
}

bool dfs(int at) {
    if (marked[at]) return false;
    marked[at] = true;
    if (at == 1) {
        answer.push_back(1);
        return true;
    }

    for (auto nxt : g[at])
        if (nxt.v != path[at].u && dfs(nxt.v)) {
            answer.push_back(at);
            return true;
        }
    return false;
}

int main() {
    int n, m;
    cin >> n >> m;
    g.resize(n);
    while (m--) {
        int u, v;
        ll d;
        cin >> u >> v >> d;
        g[u].push_back(Edge(u,v,d));
        g[v].push_back(Edge(v,u,d));
    }

    dijkstra(1);

    marked.resize(n, false);
    dfs(0);

    if (answer.size() == 0) cout << "impossible" << endl;
    else {
        cout << answer.size();
        for (int i = answer.size()-1; i >= 0; --i) cout << ' ' << answer[i];
        cout << endl;
    }

}
